@@ -143,34 +143,6 @@ BOOL WINAPI HeapDestroy( HANDLE heap /* [in] Handle of heap */ )
 }
 
 
-
-
-
-/* These are needed so that we can call the functions from inside kernel itself */
-
-/***********************************************************************
- *           HeapAlloc    (KERNEL32.@)
- */
-LPVOID WINAPI HeapAlloc( HANDLE heap, DWORD flags, SIZE_T size )
-{
-    return RtlAllocateHeap( heap, flags, size );
-}
-
-BOOL WINAPI HeapFree( HANDLE heap, DWORD flags, LPVOID ptr )
-{
-    return RtlFreeHeap( heap, flags, ptr );
-}
-
-LPVOID WINAPI HeapReAlloc( HANDLE heap, DWORD flags, LPVOID ptr, SIZE_T size )
-{
-    return RtlReAllocateHeap( heap, flags, ptr, size );
-}
-
-SIZE_T WINAPI HeapSize( HANDLE heap, DWORD flags, LPCVOID ptr )
-{
-    return RtlSizeHeap( heap, flags, ptr );
-}
-
 /*
  * Win32 Global heap functions (GlobalXXX).
  * These functions included in Win32 for compatibility with 16 bit Windows
@@ -361,10 +333,10 @@ SIZE_T WINAPI GlobalSize(HGLOBAL hmem)
    {
       retval=HeapSize(GetProcessHeap(), 0, hmem);
 
-      if (retval == ~0ul) /* It might be a GMEM_MOVEABLE data pointer */
+      if (retval == ~(SIZE_T)0) /* It might be a GMEM_MOVEABLE data pointer */
       {
           retval = HeapSize(GetProcessHeap(), 0, (char*)hmem - HGLOBAL_STORAGE);
-          if (retval != ~0ul) retval -= HGLOBAL_STORAGE;
+          if (retval != ~(SIZE_T)0) retval -= HGLOBAL_STORAGE;
       }
    }
    else
@@ -379,7 +351,7 @@ SIZE_T WINAPI GlobalSize(HGLOBAL hmem)
          else
          {
              retval = HeapSize(GetProcessHeap(), 0, (char *)pintern->Pointer - HGLOBAL_STORAGE );
-             if (retval != ~0ul) retval -= HGLOBAL_STORAGE;
+             if (retval != ~(SIZE_T)0) retval -= HGLOBAL_STORAGE;
          }
       }
       else
@@ -390,7 +362,7 @@ SIZE_T WINAPI GlobalSize(HGLOBAL hmem)
       }
       RtlUnlockHeap(GetProcessHeap());
    }
-   if (retval == ~0ul) retval = 0;
+   if (retval == ~(SIZE_T)0) retval = 0;
    return retval;
 }
 
