@@ -26,9 +26,6 @@
 #include "mfidl.h"
 #include "rpcproxy.h"
 
-#include "mfapi.h"
-#include "mferror.h"
-
 #include "mf_private.h"
 
 #include "wine/debug.h"
@@ -92,7 +89,8 @@ static ULONG WINAPI activate_object_Release(IMFActivate *iface)
 
     if (!refcount)
     {
-        activate->funcs->free_private(activate->context);
+        if (activate->funcs->free_private)
+            activate->funcs->free_private(activate->context);
         if (activate->object)
             IUnknown_Release(activate->object);
         IMFAttributes_Release(activate->attributes);
@@ -1267,44 +1265,6 @@ HRESULT WINAPI MFEnumDeviceSources(IMFAttributes *attributes, IMFActivate ***sou
     return S_OK;
 }
 
-static HRESULT evr_create_object(IMFAttributes *attributes, void *user_context, IUnknown **obj)
-{
-    FIXME("%p, %p, %p.\n", attributes, user_context, obj);
-
-    return E_NOTIMPL;
-}
-
-static void evr_shutdown_object(void *user_context, IUnknown *obj)
-{
-}
-
-static void evr_free_private(void *user_context)
-{
-}
-
-static const struct activate_funcs evr_activate_funcs =
-{
-    evr_create_object,
-    evr_shutdown_object,
-    evr_free_private,
-};
-
-HRESULT WINAPI MFCreateVideoRendererActivate(HWND hwnd, IMFActivate **activate)
-{
-    HRESULT hr;
-
-    TRACE("%p, %p.\n", hwnd, activate);
-
-    if (!activate)
-        return E_POINTER;
-
-    hr = create_activation_object(hwnd, &evr_activate_funcs, activate);
-    if (SUCCEEDED(hr))
-        IMFActivate_SetUINT64(*activate, &MF_ACTIVATE_VIDEO_WINDOW, (ULONG_PTR)hwnd);
-
-    return hr;
-}
-
 struct simple_type_handler
 {
     IMFMediaTypeHandler IMFMediaTypeHandler_iface;
