@@ -21,18 +21,18 @@ var JS_E_INVALID_WRITABLE_PROP_DESC = 0x800a13ac;
 var JS_E_NONCONFIGURABLE_REDEFINED = 0x800a13d6;
 var JS_E_NONWRITABLE_MODIFIED = 0x800a13d7;
 
-function test_date_now() {
+var tests = [];
+
+sync_test("date_now", function() {
     var now = Date.now();
     var time = (new Date()).getTime();
 
     ok(time >= now && time-now < 50, "unexpected Date.now() result " + now + " expected " + time);
 
     Date.now(1, 2, 3);
+});
 
-    next_test();
-}
-
-function test_toISOString() {
+sync_test("toISOString", function() {
     function expect(date, expected) {
         var s = date.toISOString();
         ok(s === expected, "toISOString returned " + s + " expected " + expected);
@@ -58,11 +58,9 @@ function test_toISOString() {
 
     expect_exception(function() { new Date(NaN).toISOString(); });
     expect_exception(function() { new Date(31494784780800001).toISOString(); });
+});
 
-    next_test();
-}
-
-function test_indexOf() {
+sync_test("indexOf", function() {
     function expect(array, args, exr) {
         var r = Array.prototype.indexOf.apply(array, args);
         ok(r == exr, "indexOf returned " + r + " expected " + exr);
@@ -86,11 +84,9 @@ function test_indexOf() {
     expect({"4": 4, length: 3}, [4], -1);
     expect({"test": true}, [true], -1);
     expect([1,2,3], [2, 1.9], 1);
+});
 
-    next_test();
-}
-
-function test_array_forEach() {
+sync_test("forEach", function() {
     ok(Array.prototype.forEach.length === 1, "forEach.length = " + Array.prototype.forEach.length);
 
     function test(array, expect) {
@@ -116,10 +112,14 @@ function test_array_forEach() {
         ok(this === window, "this != window");
     }, undefined);
 
-    next_test();
-}
+    var o = new Object(), a = [1,2];
+    a.forEach(function(value, index, array) {
+        ok(array === a, "array != a");
+        ok(this === o, "this != o");
+    }, o);
+});
 
-function test_isArray() {
+sync_test("isArray", function() {
     function expect_array(a, exr) {
         var r = Array.isArray(a);
         ok(r === exr, "isArray returned " + r + " expected " + exr);
@@ -133,11 +133,9 @@ function test_isArray() {
     function C() {}
     C.prototype = Array.prototype;
     expect_array(new C(), false);
+});
 
-    next_test();
-}
-
-function test_array_map() {
+sync_test("array_map", function() {
     var calls, m, arr, ctx;
 
     /* basic map call with context */
@@ -184,11 +182,26 @@ function test_array_map() {
     [1,2].map(function() {
         ok(this === window, "this != window");
     }, undefined);
+});
 
-    next_test();
-}
+sync_test("array_sort", function() {
+    var r;
 
-function test_identifier_keywords() {
+    r = [3,1,2].sort(function(x,y) { return y-x; }, 1, 2, 3, true, undefined ).join();
+    ok(r === "3,2,1", "reverse sorted [3,1,2] = " + r);
+
+    r = [3,1,2].sort(undefined).join();
+    ok(r === "1,2,3", "null sorted [3,1,2] = " + r);
+
+    try {
+        r = [3,1,2].sort(null);
+        ok(false, "expected sort(null) exception");
+    }catch(e) {
+        ok(e.name === "TypeError", "got exception " + e.name);
+    }
+});
+
+sync_test("identifier_keywords", function() {
     var o = {
         if: 1,
         default: 2,
@@ -224,9 +237,7 @@ function test_identifier_keywords() {
     ok(o.if === 1, "o.if = " + o.if);
     ok(ro().default === 2, "ro().default = " + ro().default);
     ok(o.false === true, "o.false = " + o.false);
-
-    next_test();
-}
+});
 
 function test_own_data_prop_desc(obj, prop, expected_writable, expected_enumerable,
                                  expected_configurable) {
@@ -241,7 +252,7 @@ function test_own_data_prop_desc(obj, prop, expected_writable, expected_enumerab
        + " expected " + expected_configurable);
 }
 
-function test_getOwnPropertyDescriptor() {
+sync_test("getOwnPropertyDescriptor", function() {
     var obj;
 
     obj = { test: 1 };
@@ -279,11 +290,9 @@ function test_getOwnPropertyDescriptor() {
     test_own_data_prop_desc(function(){}, "prototype", true, false, false);
     test_own_data_prop_desc(Function, "prototype", false, false, false);
     test_own_data_prop_desc(String.prototype, "constructor", true, false, true);
+});
 
-    next_test();
-}
-
-function test_defineProperty() {
+sync_test("defineProperty", function() {
     function test_accessor_prop_desc(obj, prop, orig_desc) {
         var expected_enumerable = "enumerable" in orig_desc && !!orig_desc.enumerable;
         var expected_configurable = "configurable" in orig_desc && !!orig_desc.configurable;
@@ -311,8 +320,9 @@ function test_defineProperty() {
         ok(false, "expected exception");
     }
 
-    var obj = new Object();
-    Object.defineProperty(obj, "test", {});
+    var obj = new Object(), defined;
+    defined = Object.defineProperty(obj, "test", {});
+    ok(defined === obj, "defined != obj");
     ok("test" in obj, "test is not in obj");
     test_own_data_prop_desc(obj, "test", false, false, false);
     ok(obj.test === undefined, "obj.test = " + obj.test);
@@ -503,11 +513,49 @@ function test_defineProperty() {
     test_accessor_prop_desc(obj, "no_setter", desc);
     obj.no_setter = false;
     ok(obj.no_setter === true, "no_setter = " + obj.no_setter);
+});
 
-    next_test();
-}
+sync_test("defineProperties", function() {
+    var o, defined, descs;
 
-function test_property_definitions() {
+    descs = {
+        acc_prop: { get: function() { return 1; } },
+        prop: { value: 2 },
+        e: { enumerable: true }
+    };
+
+    o = new Object();
+    defined = Object.defineProperties(o, descs);
+    ok(defined === o, "defined != o");
+    ok(o.acc_prop === 1, "o.acc_prop = " + o.acc_prop);
+    ok(o.prop === 2, "o.prop = " + o.prop);
+    ok(o.e === undefined, "o.e = " + o.e);
+    ok("e" in o, "e is not in o");
+    test_own_data_prop_desc(o, "prop", false, false, false);
+    test_own_data_prop_desc(o, "e", false, true, false);
+    for(var p in o) ok(p === "e", "p = " + p);
+
+    o = new Object();
+    Object.defineProperties(o, 1);
+    for(var p in o) ok(false, "o has property " + p);
+
+    o = Object.create(null, descs);
+    ok(o.acc_prop === 1, "o.acc_prop = " + o.acc_prop);
+    ok(o.prop === 2, "o.prop = " + o.prop);
+    ok(o.e === undefined, "o.e = " + o.e);
+    ok("e" in o, "e is not in o");
+    test_own_data_prop_desc(o, "prop", false, false, false);
+    test_own_data_prop_desc(o, "e", false, true, false);
+    for(var p in o) ok(p === "e", "p = " + p);
+
+    var desc_proto = new Object();
+    desc_proto.proto_prop = { value: true, enumerable: true };
+    descs = Object.create(desc_proto);
+    o = Object.create(null, descs);
+    ok(!("proto_prop" in o), "proto_prop is in o");
+});
+
+sync_test("property_definitions", function() {
     var obj, val, i, arr;
 
     function test_accessor_prop_desc(obj, prop, have_getter, have_setter) {
@@ -568,11 +616,9 @@ function test_property_definitions() {
     ok(obj.prop === 6, "obj.prop = " + obj.prop);
     test_accessor_prop_desc(obj, "0", true, false);
     ok(obj[0] === 7, "obj.prop = " + obj[0]);
+});
 
-    next_test();
-}
-
-function test_string_trim() {
+sync_test("string_trim", function() {
     function test_trim(value, expected) {
         var r = String.prototype.trim.call(value);
         ok(r === expected, "trim(" + value + ") = " + r);
@@ -584,11 +630,9 @@ function test_string_trim() {
     test_trim({ toString: function() { return " test "; } }, "test");
     test_trim("", "");
     test_trim(" \t\n", "");
+});
 
-    next_test();
-}
-
-function test_global_properties() {
+sync_test("global_properties", function() {
     var o;
 
     /* Make sure that global properties are not writable. */
@@ -601,20 +645,52 @@ function test_global_properties() {
     o = Infinity;
     Infinity = 1;
     ok(Infinity === o, "Infinity = " + NaN);
+});
 
-    next_test();
-}
-
-function test_string_split() {
+sync_test("string_split", function() {
     var r;
 
     /* IE9 got this wrong*/
     if("1undefined2".split(undefined).length != 1) {
         win_skip("detected broken String.prototype.split implementation");
-        next_test();
         return;
     }
 
+    function test(string, separator, result) {
+        var r = string.split(separator);
+        ok(r == result, "\"" + string + "\".split(" + separator + ") returned " + r + " expected " + result);
+    }
+
+    test("test", /^|\s+/, "test");
+    test("test", /$|\s+/, "test");
+    test("test", /^|./, "t,,,");
+    test("test", /.*/, ",");
+    test("test", /x*/, "t,e,s,t");
+    test("test", /$|x*/, "t,e,s,t");
+    test("test", /^|x*/, "t,e,s,t");
+    test("test", /t*/, ",e,s,");
+    test("xaabaax", /a*|b*/, "x,b,x");
+    test("xaabaax", /a+|b+/, "x,,,x");
+    test("xaabaax", /a+|b*/, "x,,,x");
+    test("xaaxbaax", /b+|a+/, "x,x,,x");
+    test("test", /^|t/, "tes,");
+    test("test", /^|t/, "tes,");
+    test("a,,b", /,/, "a,,b");
+    test("ab", /a*/, ",b");
+    test("aab", "a", ",,b");
+    test("a", "a", ",");
+
+    function test_length(string, separator, len) {
+        var r = string.split(separator);
+        ok(r.length === len, "\"" + string + "\".split(" + separator + ").length = "
+           + r.length + " expected " + len);
+    }
+
+    test_length("", /a*/, 0);
+    test_length("", /a+/, 1);
+    test_length("", "", 0);
+    test_length("", "x", 1);
+
     r = "1,2,3".split(undefined);
     ok(typeof(r) === "object", "typeof(r) = " + typeof(r));
     ok(r.length === 1, "r.length = " + r.length);
@@ -646,11 +722,9 @@ function test_string_split() {
     ok(typeof(r) === "object", "typeof(r) = " + typeof(r));
     ok(r.length === 1, "r.length = " + r.length);
     ok(r[0] === "", "r[0] = " + r[0]);
+});
 
-    next_test();
-}
-
-function test_getPrototypeOf() {
+sync_test("getPrototypeOf", function() {
     ok(Object.create.length === 2, "Object.create.length = " + Object.create.length);
     ok(Object.getPrototypeOf.length === 1, "Object.getPrototypeOf.length = " + Object.getPrototypeOf.length);
 
@@ -685,23 +759,193 @@ function test_getPrototypeOf() {
     obj = Object.create(null);
     ok(!("toString" in obj), "toString is in obj");
     ok(Object.getPrototypeOf(obj) === null, "Object.getPrototypeOf(obj) = " + Object.getPrototypeOf(obj));
+});
 
-    next_test();
-}
+sync_test("toString", function() {
+    var tmp, obj;
 
-var tests = [
-    test_date_now,
-    test_toISOString,
-    test_indexOf,
-    test_array_forEach,
-    test_isArray,
-    test_array_map,
-    test_identifier_keywords,
-    test_getOwnPropertyDescriptor,
-    test_defineProperty,
-    test_property_definitions,
-    test_string_trim,
-    test_global_properties,
-    test_string_split,
-    test_getPrototypeOf
-];
+    (function () { tmp = Object.prototype.toString.call(arguments); })();
+    todo_wine.
+    ok(tmp === "[object Arguments]", "toString.call(arguments) = " + tmp);
+    tmp = Object.prototype.toString.call(this);
+    todo_wine.
+    ok(tmp === "[object Window]", "toString.call(null) = " + tmp);
+    tmp = Object.prototype.toString.call(null);
+    todo_wine.
+    ok(tmp === "[object Null]", "toString.call(null) = " + tmp);
+    tmp = Object.prototype.toString.call(undefined);
+    todo_wine.
+    ok(tmp === "[object Undefined]", "toString.call(undefined) = " + tmp);
+    tmp = Object.prototype.toString.call();
+    todo_wine.
+    ok(tmp === "[object Undefined]", "toString.call() = " + tmp);
+
+    obj = Object.create(null);
+    tmp = Object.prototype.toString.call(obj);
+    ok(tmp === "[object Object]", "toString.call(Object.create(null)) = " + tmp);
+    obj = Object.create(Number.prototype);
+    tmp = Object.prototype.toString.call(obj);
+    ok(tmp === "[object Object]", "toString.call(Object.create(Number.prototype)) = " + tmp);
+});
+
+sync_test("bind", function() {
+    var f, r;
+    var o = new Object(), o2 = new Object();
+
+    f = (function() {
+        ok(this === o, "this != o");
+        ok(arguments.length === 0, "arguments.length = " + arguments.length);
+        return 1;
+    }).bind(o);
+    ok(f.length === 0, "f.length = " + f.length);
+    r = f.call(o2);
+    ok(r === 1, "r = " + r);
+
+    f = (function() {
+        ok(this === o, "this != o");
+        ok(arguments.length === 1, "arguments.length = " + arguments.length);
+        ok(arguments[0] === 1, "arguments[0] = " + arguments[0]);
+        return 1;
+    }).bind(o, 1);
+    ok(f.length === 0, "f.length = " + f.length);
+    r = f.call(o2);
+    ok(r === 1, "r = " + r);
+
+    f = (function() {
+        ok(this === o, "this != o");
+        ok(arguments.length === 2, "arguments.length = " + arguments.length);
+        ok(arguments[0] === 1, "arguments[0] = " + arguments[0]);
+        ok(arguments[1] === 2, "arguments[1] = " + arguments[0]);
+        return 1;
+    }).bind(o, 1);
+    r = f.call(o2, 2);
+    ok(r === 1, "r = " + r);
+
+    o2.f = f;
+    r = o2.f(2);
+    ok(r === 1, "r = " + r);
+
+    f = (function test(x, y, z) {
+        ok(this === o, "this != o");
+        ok(arguments.length === 2, "arguments.length = " + arguments.length);
+        ok(x === 1, "x = " + x);
+        ok(y === 2, "y = " + y);
+        ok(z === undefined, "z = " + z);
+        return 1;
+    }).bind(o, 1);
+    ok(f.length === 2, "f.length = " + f.length);
+    r = f.call(o2, 2);
+    ok(r === 1, "r = " + r);
+    ok(f.toString() === "\nfunction() {\n    [native code]\n}\n", "f.toString() = " + f.toString());
+    ok(!("prototype" in f), "bound function has prototype");
+
+    var a = [];
+    f = Array.prototype.push.bind(a, 1);
+    f();
+    ok(a.length === 1, "a.length = " + a.length);
+    f(2);
+    ok(a.length === 3, "a.length = " + a.length);
+    ok(f.length === 0, "f.length = " + f.length);
+    ok(f.toString() === "\nfunction() {\n    [native code]\n}\n", "f.toString() = " + f.toString());
+    ok(a.toString() === "1,1,2", "a = " + a);
+    f.call([], 3);
+    ok(a.toString() === "1,1,2,1,3", "a = " + a);
+
+    f = (function() { return this; }).bind(a);
+    ok(f() === a, "f() != a");
+
+    f = (function() { return this; }).bind(null);
+    ok(f() === window, "f() = " + f());
+
+    var t;
+    f = (function() { return t = this; }).bind(a);
+    ok(new f() === t, "new f() != a");
+    ok(typeof(t) === "object", "typeof(t) = " + typeof(t));
+    ok(t != a, "t == a");
+
+    ok(Function.prototype.bind.length === 1, "Function.prototype.bind.length = " + Function.prototype.bind.length);
+});
+
+sync_test("keys", function() {
+    var o = { a: 1, b: 2, c: 3 };
+    var keys = Object.keys(o).sort().join();
+    ok(keys === "a,b,c", "keys = " + keys);
+
+    o = Object.create(o);
+    keys = Object.keys(o).sort().join();
+    ok(keys === "", "keys = " + keys);
+
+    o.test = 1;
+    keys = Object.keys(o).sort().join();
+    ok(keys === "test", "keys = " + keys);
+
+    ok(Object.keys.length === 1, "Object.keys.length = " + Object.keys.length);
+});
+
+sync_test("reduce", function() {
+    var r, array;
+
+    r = [1,2,3].reduce(function(a, value) { return a + value + 10; });
+    ok(r === 26, "reduce() returned " + r);
+
+    r = [1,2,3].reduce(function(a, value) { return a + value + 10; }, 1);
+    ok(r === 37, "reduce() returned " + r);
+
+    r = [1,2,3].reduce(function(a, value) { return a + value; }, "str");
+    ok(r === "str123", "reduce() returned " + r);
+
+    array = [1,2,3];
+    r = array.reduce(function(a, value, index, src) {
+        ok(src === array, "src != array");
+        return a + "(" + index + "," + value + ")";
+    }, "str");
+    ok(r === "str(0,1)(1,2)(2,3)", "reduce() returned " + r);
+
+    r = [1,2,3].reduce(function(a, value, index, src) {
+        src[0] = false;
+        delete src[1];
+        src[2] = "test";
+        return a + value;
+    }, "");
+    ok(r === "1test", "reduce() returned " + r);
+
+    r = [1].reduce(function(a) { return 0; });
+    ok(r === 1, "[1].reduce() returned " + r);
+
+    r = [1].reduce(function(a) { return 0; }, 2);
+    ok(r === 0, "[1].reduce(2) returned " + r);
+
+    r = [].reduce(function(a) { return 0; }, 2);
+    ok(r === 2, "[].reduce(2) returned " + r);
+
+    r = [].reduce(function(a) { return 0; }, undefined);
+    ok(r === undefined, "[].reduce(undefined) returned " + r);
+
+    try {
+        [].reduce(function(a) { return 0; });
+        ok(false, "expected exception");
+    }catch(e) {}
+
+    ok(Array.prototype.reduce.length === 1, "Array.prototype.reduce.length = " + Array.prototype.reduce.length);
+});
+
+sync_test("preventExtensions", function() {
+    var o = {};
+    var r = Object.preventExtensions(o);
+    ok(r === o, "r != o");
+    o.x = 1;
+    todo_wine.
+    ok(!("x" in o), "x property added to o");
+    try {
+        Object.defineProperty(o, "y", { value: true });
+        todo_wine.
+        ok(false, "expected exception");
+    }catch(e) {
+        ok(e.name === "TypeError", "got " + e.name + " exception");
+    }
+
+    r = Object.preventExtensions(o);
+    ok(r === o, "r != o");
+
+    ok(Object.preventExtensions.length === 1, "Object.preventExtensions.length = " + Object.preventExtensions.length);
+});
