@@ -708,6 +708,7 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
             switch (func->u.s.CR)
             {
             case 3:
+                len++; /* mov x29,sp */
                 len++; /* stp x29,lr,[sp,0] */
                 if (local_size <= 512) break;
                 /* fall through */
@@ -717,9 +718,10 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
                 if (local_size > 4088) len++;  /* sub sp,sp,#4088 */
                 break;
             }
-            if (offset < len + 4 * func->u.s.H)  /* prolog */
+            len += 4 * func->u.s.H;
+            if (offset < len)  /* prolog */
             {
-                skip = len + 4 * func->u.s.H - offset;
+                skip = len - offset;
             }
             else if (offset >= func->u.s.FunctionLength - (len + 1))  /* epilog */
             {
@@ -733,6 +735,7 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         if (func->u.s.CR == 3)
         {
             DWORD64 *fp = (DWORD64 *) context->u.s.Fp; /* u.X[29] */
+            context->Sp = context->u.s.Fp;
             context->u.X[29] = fp[0];
             context->u.X[30] = fp[1];
         }
@@ -748,34 +751,36 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         switch (func->u.s.CR)
         {
         case 3:
+            /* mov x29,sp */
+            if (pos++ >= skip) context->Sp = context->u.s.Fp;
             if (local_size <= 512)
             {
                 /* stp x29,lr,[sp,-#local_size]! */
-                if (pos++ > skip) restore_regs( 29, 2, -local_size_regs, context, ptrs );
+                if (pos++ >= skip) restore_regs( 29, 2, -local_size_regs, context, ptrs );
                 break;
             }
             /* stp x29,lr,[sp,0] */
-            if (pos++ > skip) restore_regs( 29, 2, 0, context, ptrs );
+            if (pos++ >= skip) restore_regs( 29, 2, 0, context, ptrs );
             /* fall through */
         case 0:
         case 1:
             if (!local_size) break;
             /* sub sp,sp,#local_size */
-            if (pos++ > skip) context->Sp += (local_size - 1) % 4088 + 1;
-            if (local_size > 4088 && pos++ > skip) context->Sp += 4088;
+            if (pos++ >= skip) context->Sp += (local_size - 1) % 4088 + 1;
+            if (local_size > 4088 && pos++ >= skip) context->Sp += 4088;
             break;
         }
 
-        if (func->u.s.H && offset < len + 4) pos += 4;
+        if (func->u.s.H) pos += 4;
 
         if (fp_size)
         {
-            if (func->u.s.RegF % 2 == 0 && pos++ > skip)
+            if (func->u.s.RegF % 2 == 0 && pos++ >= skip)
                 /* str d%u,[sp,#fp_size] */
                 restore_fpregs( 8 + func->u.s.RegF, 1, int_regs + fp_regs - 1, context, ptrs );
-            for (i = func->u.s.RegF / 2 - 1; i >= 0; i--)
+            for (i = (func->u.s.RegF + 1) / 2 - 1; i >= 0; i--)
             {
-                if (pos++ <= skip) continue;
+                if (pos++ < skip) continue;
                 if (!i && !int_size)
                      /* stp d8,d9,[sp,-#regsave]! */
                     restore_fpregs( 8, 2, -saved_regs, context, ptrs );
@@ -785,9 +790,9 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
             }
         }
 
-        if (pos++ > skip)
+        if (func->u.s.RegI % 2)
         {
-            if (func->u.s.RegI % 2)
+            if (pos++ >= skip)
             {
                 /* stp xn,lr,[sp,#offset] */
                 if (func->u.s.CR == 1) restore_regs( 30, 1, int_regs - 1, context, ptrs );
@@ -796,14 +801,16 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
                               (func->u.s.RegI > 1) ? func->u.s.RegI - 1 : -saved_regs,
                               context, ptrs );
             }
-            else if (func->u.s.CR == 1)
-                /* str lr,[sp,#offset] */
-                restore_regs( 30, 1, func->u.s.RegI ? int_regs - 1 : -saved_regs, context, ptrs );
+        }
+        else if (func->u.s.CR == 1)
+        {
+            /* str lr,[sp,#offset] */
+            if (pos++ >= skip) restore_regs( 30, 1, func->u.s.RegI ? int_regs - 1 : -saved_regs, context, ptrs );
         }
 
-        for (i = func->u.s.RegI / 2 - 1; i >= 0; i--)
+        for (i = func->u.s.RegI/ 2 - 1; i >= 0; i--)
         {
-            if (pos++ <= skip) continue;
+            if (pos++ < skip) continue;
             if (i)
                 /* stp xn,xn+1,[sp,#offset] */
                 restore_regs( 19 + 2 * i, 2, 2 * i, context, ptrs );
@@ -940,9 +947,7 @@ PVOID WINAPI RtlVirtualUnwind( ULONG type, ULONG_PTR base, ULONG_PTR pc,
  * else. All CFI instructions are either DW_CFA_def_cfa_expression or
  * DW_CFA_expression, and the expressions have the following format:
  *
- * DW_OP_breg29; sleb128 0x10           | Load x29 + 0x10
- * DW_OP_deref                          | Get *(x29 + 0x10) == context
- * DW_OP_plus_uconst; uleb128 <OFFSET>  | Add offset to get struct member
+ * DW_OP_breg31; sleb128 <OFFSET>       | Load x31 + struct member offset
  * [DW_OP_deref]                        | Dereference, only for CFA
  */
 extern void * WINAPI call_consolidate_callback( CONTEXT *context,
@@ -950,40 +955,60 @@ extern void * WINAPI call_consolidate_callback( CONTEXT *context,
                                                 EXCEPTION_RECORD *rec,
                                                 TEB *teb );
 __ASM_GLOBAL_FUNC( call_consolidate_callback,
-                   "stp x29, x30, [sp, #-0x20]!\n\t"
-                   __ASM_CFI(".cfi_def_cfa_offset 32\n\t")
-                   __ASM_CFI(".cfi_offset 29, -32\n\t")
-                   __ASM_CFI(".cfi_offset 30, -24\n\t")
+                   "stp x29, x30, [sp, #-0x30]!\n\t"
+                   __ASM_CFI(".cfi_def_cfa_offset 48\n\t")
+                   __ASM_CFI(".cfi_offset 29, -48\n\t")
+                   __ASM_CFI(".cfi_offset 30, -40\n\t")
+                   __ASM_SEH(".seh_nop\n\t")
+                   "stp x1,  x2,  [sp, #0x10]\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
+                   "str x3,       [sp, #0x20]\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
                    "mov x29, sp\n\t"
                    __ASM_CFI(".cfi_def_cfa_register 29\n\t")
-                   "str x0, [sp, 0x10]\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
                    __ASM_CFI(".cfi_remember_state\n\t")
-                   __ASM_CFI(".cfi_escape 0x0f,0x07,0x8d,0x10,0x06,0x23,0x80,0x02,0x06\n\t") /* CFA */
-                   __ASM_CFI(".cfi_escape 0x10,0x13,0x06,0x8d,0x10,0x06,0x23,0xa0,0x01\n\t") /* x19 */
-                   __ASM_CFI(".cfi_escape 0x10,0x14,0x06,0x8d,0x10,0x06,0x23,0xa8,0x01\n\t") /* x20 */
-                   __ASM_CFI(".cfi_escape 0x10,0x15,0x06,0x8d,0x10,0x06,0x23,0xb0,0x01\n\t") /* x21 */
-                   __ASM_CFI(".cfi_escape 0x10,0x16,0x06,0x8d,0x10,0x06,0x23,0xb8,0x01\n\t") /* x22 */
-                   __ASM_CFI(".cfi_escape 0x10,0x17,0x06,0x8d,0x10,0x06,0x23,0xc0,0x01\n\t") /* x23 */
-                   __ASM_CFI(".cfi_escape 0x10,0x18,0x06,0x8d,0x10,0x06,0x23,0xc8,0x01\n\t") /* x24 */
-                   __ASM_CFI(".cfi_escape 0x10,0x19,0x06,0x8d,0x10,0x06,0x23,0xd0,0x01\n\t") /* x25 */
-                   __ASM_CFI(".cfi_escape 0x10,0x1a,0x06,0x8d,0x10,0x06,0x23,0xd8,0x01\n\t") /* x26 */
-                   __ASM_CFI(".cfi_escape 0x10,0x1b,0x06,0x8d,0x10,0x06,0x23,0xe0,0x01\n\t") /* x27 */
-                   __ASM_CFI(".cfi_escape 0x10,0x1c,0x06,0x8d,0x10,0x06,0x23,0xe8,0x01\n\t") /* x28 */
-                   __ASM_CFI(".cfi_escape 0x10,0x1d,0x06,0x8d,0x10,0x06,0x23,0xf0,0x01\n\t") /* x29 */
-                   __ASM_CFI(".cfi_escape 0x10,0x1e,0x06,0x8d,0x10,0x06,0x23,0xf8,0x01\n\t") /* x30 */
-                   __ASM_CFI(".cfi_escape 0x10,0x48,0x06,0x8d,0x10,0x06,0x23,0x90,0x03\n\t") /* d8  */
-                   __ASM_CFI(".cfi_escape 0x10,0x49,0x06,0x8d,0x10,0x06,0x23,0xa0,0x03\n\t") /* d9  */
-                   __ASM_CFI(".cfi_escape 0x10,0x4a,0x06,0x8d,0x10,0x06,0x23,0xb0,0x03\n\t") /* d10 */
-                   __ASM_CFI(".cfi_escape 0x10,0x4b,0x06,0x8d,0x10,0x06,0x23,0xc0,0x03\n\t") /* d11 */
-                   __ASM_CFI(".cfi_escape 0x10,0x4c,0x06,0x8d,0x10,0x06,0x23,0xd0,0x03\n\t") /* d12 */
-                   __ASM_CFI(".cfi_escape 0x10,0x4d,0x06,0x8d,0x10,0x06,0x23,0xe0,0x03\n\t") /* d13 */
-                   __ASM_CFI(".cfi_escape 0x10,0x4e,0x06,0x8d,0x10,0x06,0x23,0xf0,0x03\n\t") /* d14 */
-                   __ASM_CFI(".cfi_escape 0x10,0x4f,0x06,0x8d,0x10,0x06,0x23,0x80,0x04\n\t") /* d15 */
+                   /* Memcpy the context onto the stack */
+                   "sub sp, sp, #0x390\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
+                   "mov x1,  x0\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
+                   "mov x0,  sp\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
+                   "mov x2,  #0x390\n\t"
+                   __ASM_SEH(".seh_nop\n\t")
+                   "bl " __ASM_NAME("memcpy") "\n\t"
+                   __ASM_CFI(".cfi_def_cfa 31, 0\n\t")
+                   __ASM_CFI(".cfi_escape 0x0f,0x04,0x8f,0x80,0x02,0x06\n\t") /* CFA, DW_OP_breg31 + 0x100, DW_OP_deref */
+                   __ASM_CFI(".cfi_escape 0x10,0x13,0x03,0x8f,0xa0,0x01\n\t") /* x19, DW_OP_breg31 + 0xA0 */
+                   __ASM_CFI(".cfi_escape 0x10,0x14,0x03,0x8f,0xa8,0x01\n\t") /* x20 */
+                   __ASM_CFI(".cfi_escape 0x10,0x15,0x03,0x8f,0xb0,0x01\n\t") /* x21 */
+                   __ASM_CFI(".cfi_escape 0x10,0x16,0x03,0x8f,0xb8,0x01\n\t") /* x22 */
+                   __ASM_CFI(".cfi_escape 0x10,0x17,0x03,0x8f,0xc0,0x01\n\t") /* x23 */
+                   __ASM_CFI(".cfi_escape 0x10,0x18,0x03,0x8f,0xc8,0x01\n\t") /* x24 */
+                   __ASM_CFI(".cfi_escape 0x10,0x19,0x03,0x8f,0xd0,0x01\n\t") /* x25 */
+                   __ASM_CFI(".cfi_escape 0x10,0x1a,0x03,0x8f,0xd8,0x01\n\t") /* x26 */
+                   __ASM_CFI(".cfi_escape 0x10,0x1b,0x03,0x8f,0xe0,0x01\n\t") /* x27 */
+                   __ASM_CFI(".cfi_escape 0x10,0x1c,0x03,0x8f,0xe8,0x01\n\t") /* x28 */
+                   __ASM_CFI(".cfi_escape 0x10,0x1d,0x03,0x8f,0xf0,0x01\n\t") /* x29 */
+                   __ASM_CFI(".cfi_escape 0x10,0x1e,0x03,0x8f,0xf8,0x01\n\t") /* x30 */
+                   __ASM_CFI(".cfi_escape 0x10,0x48,0x03,0x8f,0x90,0x03\n\t") /* d8  */
+                   __ASM_CFI(".cfi_escape 0x10,0x49,0x03,0x8f,0x98,0x03\n\t") /* d9  */
+                   __ASM_CFI(".cfi_escape 0x10,0x4a,0x03,0x8f,0xa0,0x03\n\t") /* d10 */
+                   __ASM_CFI(".cfi_escape 0x10,0x4b,0x03,0x8f,0xa8,0x03\n\t") /* d11 */
+                   __ASM_CFI(".cfi_escape 0x10,0x4c,0x03,0x8f,0xb0,0x03\n\t") /* d12 */
+                   __ASM_CFI(".cfi_escape 0x10,0x4d,0x03,0x8f,0xb8,0x03\n\t") /* d13 */
+                   __ASM_CFI(".cfi_escape 0x10,0x4e,0x03,0x8f,0xc0,0x03\n\t") /* d14 */
+                   __ASM_CFI(".cfi_escape 0x10,0x4f,0x03,0x8f,0xc8,0x03\n\t") /* d15 */
+                   __ASM_SEH(".seh_context\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
+                   "ldp x1,  x2,  [x29, #0x10]\n\t"
+                   "ldr x18,      [x29, #0x20]\n\t"
                    "mov x0,  x2\n\t"
-                   "mov x18, x3\n\t"
                    "blr x1\n\t"
+                   "mov sp,  x29\n\t"
                    __ASM_CFI(".cfi_restore_state\n\t")
-                   "ldp x29, x30, [sp], #32\n\t"
+                   "ldp x29, x30, [sp], #48\n\t"
                    __ASM_CFI(".cfi_restore 30\n\t")
                    __ASM_CFI(".cfi_restore 29\n\t")
                    __ASM_CFI(".cfi_def_cfa 31, 0\n\t")
@@ -1188,6 +1213,9 @@ void WINAPI RtlUnwind( void *frame, void *target_ip, EXCEPTION_RECORD *rec, void
 __ASM_STDCALL_FUNC( RtlRaiseException, 4,
                    "sub sp, sp, #0x3b0\n\t" /* 0x390 (context) + 0x20 */
                    "stp x29, x30, [sp]\n\t"
+                   __ASM_SEH(".seh_stackalloc 0x3b0\n\t")
+                   __ASM_SEH(".seh_save_fplr 0\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    __ASM_CFI(".cfi_def_cfa x29, 944\n\t")
                    __ASM_CFI(".cfi_offset x30, -936\n\t")
                    __ASM_CFI(".cfi_offset x29, -944\n\t")
