@@ -1244,7 +1244,9 @@ void WINAPI RtlUnwindEx( PVOID end_frame, PVOID target_ip, EXCEPTION_RECORD *rec
             {
                 ULONG64 frame;
 
-                *context = new_context = *dispatch.ContextRecord;
+                new_context = *dispatch.ContextRecord;
+                new_context.ContextFlags &= ~0x40;
+                *context = new_context;
                 dispatch.ContextRecord = context;
                 RtlVirtualUnwind( UNW_FLAG_NHANDLER, dispatch.ImageBase,
                         dispatch.ControlPc, dispatch.FunctionEntry,
@@ -1267,7 +1269,9 @@ void WINAPI RtlUnwindEx( PVOID end_frame, PVOID target_ip, EXCEPTION_RECORD *rec
 
                     teb_frame = __wine_pop_frame( teb_frame );
 
-                    *context = new_context = *dispatch.ContextRecord;
+                    new_context = *dispatch.ContextRecord;
+                    new_context.ContextFlags &= ~0x40;
+                    *context = new_context;
                     dispatch.ContextRecord = context;
                     RtlVirtualUnwind( UNW_FLAG_NHANDLER, dispatch.ImageBase,
                             dispatch.ControlPc, dispatch.FunctionEntry,
