@@ -87,7 +87,6 @@
 #include "winternl.h"
 #include "unix_private.h"
 #include "wine/list.h"
-#include "wine/library.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(module);
@@ -99,7 +98,7 @@ NTSTATUS (WINAPI *pKiUserExceptionDispatcher)(EXCEPTION_RECORD*,CONTEXT*) = NULL
 void     (WINAPI *pLdrInitializeThunk)(CONTEXT*,void**,ULONG_PTR,ULONG_PTR) = NULL;
 void     (WINAPI *pRtlUserThreadStart)( PRTL_THREAD_START_ROUTINE entry, void *arg ) = NULL;
 
-static void (CDECL *p__wine_set_unix_funcs)( int version, const struct unix_funcs *funcs );
+static NTSTATUS (CDECL *p__wine_set_unix_funcs)( int version, const struct unix_funcs *funcs );
 
 #ifdef __GNUC__
 static void fatal_error( const char *err, ... ) __attribute__((noreturn, format(printf,1,2)));
@@ -610,7 +609,7 @@ static NTSTATUS map_so_dll( const IMAGE_NT_HEADERS *nt_descr, HMODULE module )
                   + sizeof(IMAGE_NT_HEADERS)
                   + nb_sections * sizeof(IMAGE_SECTION_HEADER));
 
-    if (wine_anon_mmap( addr, size, PROT_READ | PROT_WRITE, MAP_FIXED ) != addr) return STATUS_NO_MEMORY;
+    if (anon_mmap_fixed( addr, size, PROT_READ | PROT_WRITE, 0 ) != addr) return STATUS_NO_MEMORY;
 
     dos = (IMAGE_DOS_HEADER *)addr;
     nt  = (IMAGE_NT_HEADERS *)((BYTE *)(dos + 1) + sizeof(builtin_signature));
@@ -815,6 +814,48 @@ static void load_builtin_callback( void *module, const char *filename )
     callback_module = module;
 }
 
+
+/***********************************************************************
+ *           load_libwine
+ */
+static void load_libwine(void)
+{
+#ifdef __APPLE__
+#define LIBWINE "libwine.1.dylib"
+#elif defined(__ANDROID__)
+#define LIBWINE "libwine.so"
+#else
+#define LIBWINE "libwine.so.1"
+#endif
+    typedef void (*load_dll_callback_t)( void *, const char * );
+    static void (*p_wine_dll_set_callback)( load_dll_callback_t load );
+    static int *p___wine_main_argc;
+    static char ***p___wine_main_argv;
+    static char ***p___wine_main_environ;
+    static WCHAR ***p___wine_main_wargv;
+
+    char *path;
+    void *handle;
+
+    if (build_dir) path = build_path( build_dir, "libs/wine/" LIBWINE );
+    else path = build_path( dll_dir, "../" LIBWINE );
+
+    if (!(handle = dlopen( path, RTLD_NOW )) && !(handle = dlopen( LIBWINE, RTLD_NOW ))) return;
+
+    p_wine_dll_set_callback = dlsym( handle, "wine_dll_set_callback" );
+    p___wine_main_argc      = dlsym( handle, "__wine_main_argc" );
+    p___wine_main_argv      = dlsym( handle, "__wine_main_argv" );
+    p___wine_main_wargv     = dlsym( handle, "__wine_main_wargv" );
+    p___wine_main_environ   = dlsym( handle, "__wine_main_environ" );
+
+    if (p_wine_dll_set_callback) p_wine_dll_set_callback( load_builtin_callback );
+    if (p___wine_main_argc) *p___wine_main_argc = main_argc;
+    if (p___wine_main_argv) *p___wine_main_argv = main_argv;
+    if (p___wine_main_wargv) *p___wine_main_wargv = main_wargv;
+    if (p___wine_main_environ) *p___wine_main_environ = main_envp;
+}
+
+
 /***********************************************************************
  *           dlopen_dll
  */
@@ -1367,8 +1408,6 @@ static struct unix_funcs unix_funcs =
     get_unix_codepage_data,
     get_locales,
     virtual_release_address_space,
-    exec_process,
-    server_init_process_done,
     set_show_dot_files,
     load_so_dll,
     load_builtin_dll,
@@ -1388,6 +1427,7 @@ static struct unix_funcs unix_funcs =
 static void start_main_thread(void)
 {
     BOOL suspend;
+    NTSTATUS status;
     TEB *teb = virtual_alloc_first_teb();
 
     signal_init_threading();
@@ -1400,48 +1440,19 @@ static void start_main_thread(void)
     init_cpu_info();
     init_files();
     NtCreateKeyedEvent( &keyed_event, GENERIC_READ | GENERIC_WRITE, NULL, 0 );
-    p__wine_set_unix_funcs( NTDLL_UNIXLIB_VERSION, &unix_funcs );
+    status = p__wine_set_unix_funcs( NTDLL_UNIXLIB_VERSION, &unix_funcs );
+    if (status) exec_process( status );
+    server_init_process_done();
 }
 
 
 #ifdef __APPLE__
-struct apple_stack_info
-{
-    void *stack;
-    size_t desired_size;
-};
-
 static void *apple_wine_thread( void *arg )
 {
     start_main_thread();
     return NULL;
 }
 
-/***********************************************************************
- *           apple_alloc_thread_stack
- *
- * Callback for mmap_enum_reserved_areas to allocate space for
- * the secondary thread's stack.
- */
-#ifndef _WIN64
-static int CDECL apple_alloc_thread_stack( void *base, size_t size, void *arg )
-{
-    struct apple_stack_info *info = arg;
-
-    /* For mysterious reasons, putting the thread stack at the very top
-     * of the address space causes subsequent execs to fail, even on the
-     * child side of a fork.  Avoid the top 16MB. */
-    char * const limit = (char*)0xff000000;
-    if ((char *)base >= limit) return 0;
-    if (size > limit - (char*)base)
-        size = limit - (char*)base;
-    if (size < info->desired_size) return 0;
-    info->stack = wine_anon_mmap( (char *)base + size - info->desired_size,
-                                  info->desired_size, PROT_READ|PROT_WRITE, MAP_FIXED );
-    return (info->stack != (void *)-1);
-}
-#endif
-
 /***********************************************************************
  *           apple_create_wine_thread
  *
@@ -1452,33 +1463,13 @@ static int CDECL apple_alloc_thread_stack( void *base, size_t size, void *arg )
  */
 static void apple_create_wine_thread( void *arg )
 {
-    int success = 0;
     pthread_t thread;
     pthread_attr_t attr;
 
-    if (!pthread_attr_init( &attr ))
-    {
-#ifndef _WIN64
-        struct apple_stack_info info;
-
-        /* Try to put the new thread's stack in the reserved area.  If this
-         * fails, just let it go wherever.  It'll be a waste of space, but we
-         * can go on. */
-        if (!pthread_attr_getstacksize( &attr, &info.desired_size ) &&
-            mmap_enum_reserved_areas( apple_alloc_thread_stack, &info, 1 ))
-        {
-            mmap_remove_reserved_area( info.stack, info.desired_size );
-            pthread_attr_setstackaddr( &attr, (char*)info.stack + info.desired_size );
-        }
-#endif
-
-        if (!pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE ) &&
-            !pthread_create( &thread, &attr, apple_wine_thread, NULL ))
-            success = 1;
-
-        pthread_attr_destroy( &attr );
-    }
-    if (!success) exit(1);
+    pthread_attr_init( &attr );
+    pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );
+    if (pthread_create( &thread, &attr, apple_wine_thread, NULL )) exit(1);
+    pthread_attr_destroy( &attr );
 }
 
 
@@ -1660,7 +1651,7 @@ void __wine_main( int argc, char *argv[], char *envp[] )
     load_ntdll();
 
     init_environment( argc, argv, envp );
-    wine_dll_set_callback( load_builtin_callback );
+    load_libwine();
 
 #ifdef __APPLE__
     apple_main_thread();
