@@ -63,7 +63,6 @@ extern char **environ;
 #include "windef.h"
 #include "winbase.h"
 #include "wine/asm.h"
-#include "wine/library.h"
 
 /* argc/argv for the Windows application */
 int __wine_main_argc = 0;
@@ -92,14 +91,18 @@ static int nb_dlls;
 
 static const IMAGE_NT_HEADERS *main_exe;
 
+typedef void (*load_dll_callback_t)( void *, const char * );
 static load_dll_callback_t load_dll_callback;
 
-static const char *build_dir;
+extern const char *build_dir;
 static const char *default_dlldir;
 static const char **dll_paths;
 static unsigned int nb_dll_paths;
 static int dll_path_maxlen;
 
+extern void *wine_anon_mmap( void *start, size_t size, int prot, int flags );
+extern void wine_init_argv0_path( const char *argv0 );
+extern void wine_init( int argc, char *argv[], char *error, int error_size );
 extern void mmap_init(void);
 extern const char *get_dlldir( const char **default_dlldir );
 
@@ -132,7 +135,7 @@ static void build_dll_path(void)
         dll_path_maxlen = strlen(dlldir);
         dll_paths[nb_dll_paths++] = dlldir;
     }
-    else if ((build_dir = wine_get_build_dir()))
+    else if (build_dir)
     {
         dll_path_maxlen = strlen(build_dir) + sizeof("/programs");
     }
@@ -494,20 +497,20 @@ void wine_dll_set_callback( load_dll_callback_t load )
 }
 
 
+#ifdef __ASM_OBSOLETE
+
 /***********************************************************************
  *           wine_dll_enum_load_path
  *
  * Enumerate the dll load path.
  */
-const char *wine_dll_enum_load_path( unsigned int index )
+const char *wine_dll_enum_load_path_obsolete( unsigned int index )
 {
     if (index >= nb_dll_paths) return NULL;
     return dll_paths[index];
 }
 
 
-#ifdef __ASM_OBSOLETE
-
 /*
  * These functions provide wrappers around dlopen() and associated
  * functions.  They work around a bug in glibc 2.1.x where calling
@@ -789,6 +792,7 @@ int wine_dll_get_owner_obsolete( const char *name, char *buffer, int size, int *
 __ASM_OBSOLETE(wine_dlopen);
 __ASM_OBSOLETE(wine_dlsym);
 __ASM_OBSOLETE(wine_dlclose);
+__ASM_OBSOLETE(wine_dll_enum_load_path);
 __ASM_OBSOLETE(wine_dll_get_owner);
 __ASM_OBSOLETE(wine_dll_load);
 __ASM_OBSOLETE(wine_dll_load_main_exe);
