@@ -42,7 +42,6 @@
 # include <stdint.h>
 #endif
 
-#include "wine/library.h"
 #include "wine/list.h"
 
 struct reserved_area
@@ -67,6 +66,8 @@ static const unsigned int granularity_mask = 0xffff;  /* reserved areas have 64k
 #define MAP_ANON 0
 #endif
 
+void wine_mmap_add_reserved_area( void *addr, size_t size );
+
 static inline int get_fdzero(void)
 {
     static int fd = -1;
@@ -215,7 +216,7 @@ void *wine_anon_mmap( void *start, size_t size, int prot, int flags )
         /* If available, this will attempt a fixed mapping in-kernel */
         flags |= MAP_TRYFIXED;
 #elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
-        if ( start && mmap( start, size, prot, flags | MAP_FIXED | MAP_EXCL, get_fdzero(), 0 ) )
+        if ( start && mmap( start, size, prot, flags | MAP_FIXED | MAP_EXCL, get_fdzero(), 0 ) != MAP_FAILED )
             return start;
 #elif defined(__svr4__) || defined(__NetBSD__) || defined(__APPLE__)
         if ( try_mmap_fixed( start, size, prot, flags, get_fdzero(), 0 ) )
